from textwrap import dedent

import pytest
from markdown import Markdown

from rst_in_md import RestructuredTextInMarkdownPreProcessor


@pytest.fixture()
def md():
    return Markdown(extensions=["rst_in_md", "fenced_code", "attr_list"])


def test_load_extension(md):
    converted = md.convert(source=(source := ""))
    assert source == converted

    assert any(
        isinstance(p, RestructuredTextInMarkdownPreProcessor) for p in md.preprocessors
    )


def test_higher_priority_than_html_block(md):
    rst_index = md.preprocessors.get_index_for_name("rst-in-md")
    html_index = md.preprocessors.get_index_for_name("html_block")
    assert rst_index < html_index


def test_higher_priority_than_fenced_code(md):
    rst_index = md.preprocessors.get_index_for_name("rst-in-md")
    fenced_code_index = md.preprocessors.get_index_for_name("fenced_code_block")
    assert rst_index < fenced_code_index


def test_fenced_code_blocks(md):
    languages = [
        "rst",
        "rest",
        "restructuredtext",
        "{.rst}",
        "{.rest}",
        "{.restructuredtext}",
        "{.rst attr=value}",
        "{ .rst }",
        "{ .rst attr=value }",
    ]
    for lang in languages:
        source = dedent(f"""
        ```{lang}
        test
        ```
        """).strip("\n")
        assert md.convert(source=source) == "<p>\n  test\n</p>"


def test_ignore_rst_in_md(md):
    source = dedent("""
    ```{.rst rst-in-md=false}
    Section to ignore
    ```
    """).strip("\n")
    assert (
        md.convert(source=source)
        == '<pre><code class="language-rst" rst-in-md="false">Section to ignore\n</code></pre>'  # noqa: E501
    )



"""Module to convert restructured text to html in markdown."""

import warnings

from markdown import Markdown
from markdown.extensions.attr_list import get_attrs_and_remainder
from markdown.extensions.fenced_code import FencedBlockPreprocessor
from markdown.preprocessors import Preprocessor

from rst_in_md.conversion import BS4_FORMATTER, LANGUAGES, rst_to_soup


class RestructuredTextInMarkdownPreProcessor(Preprocessor):
    """Preprocessor to convert restructured text to html in markdown."""

    _Processor = FencedBlockPreprocessor(Markdown(), {})
    FENCED_BLOCK_RE = _Processor.FENCED_BLOCK_RE
    handle_attrs = _Processor.handle_attrs

    def run(self, lines: list[str]) -> list[str]:
        """Strip restuctured text from markdown and replace it with converted html.

        This method will look for fenced code blocks in markdown that are marked as
        restructured text (`rst`, `rest`, `restructuredtext`) and convert them to html.
        It leverages the same regex as the [FencedBlockPreprocessor](https://github.com/Python-Markdown/markdown/blob/33359faa385f59b84cd87df5f4b0996055a482e2/markdown/extensions/fenced_code.py#L56-L67)
        to find the blocks.

        You can also ignore a block from being converted by `rst-in-md` adding
        `rst-in-md=false` to the attributes: ````{.rst rst-in-md=false}`. You can see an
        example of this [here](../guides/inline_ignore.md).

        Args:
            lines (list[str]): List of lines in markdown.

        Returns:
            list[str]: List of lines in markdown with rst replaced with html.
        """
        text = "\n".join(lines)
        processed = ""
        index = 0
        for match in self.FENCED_BLOCK_RE.finditer(text):
            lang = None
            attrs = []
            classes = []
            config = {}
            if match.group("lang") is not None:
                lang = match.group("lang")
            elif match.group("attrs") is not None:
                attrs, remainder = get_attrs_and_remainder(match.group("attrs"))
                if remainder:  # skip this match due to invalid syntax
                    warnings.warn("Invalid syntax parsing attributes", stacklevel=1)
                    processed += text[match.start() : match.end()]
                    index = match.end()
                    continue
                _, classes, config = self.handle_attrs(attrs)
                if len(classes) > 0:
                    lang = classes.pop(0)

            processed += text[index : match.start()]

            if lang not in LANGUAGES or config.get("rst-in-md") == "false":
                processed += text[match.start() : match.end()]
            else:
                try:
                    processed += rst_to_soup(match.group("code")).prettify(
                        formatter=BS4_FORMATTER,
                    )
                except ValueError as e:
                    warnings.warn(str(e), stacklevel=1)
                    processed += text[match.start() : match.end()]

            index = match.end()
        processed += text[index:]
        return processed.split("\n")



import sys
from textwrap import dedent
from unittest.mock import patch

import pytest
from markdown import Markdown
from pymdownx.superfences import SuperFencesCodeExtension

from rst_in_md import superfence_formatter, superfence_validator


@pytest.fixture()
def md():
    return Markdown(
        extensions=[
            "rst_in_md",
            "admonition",
            "attr_list",
            "pymdownx.superfences",
        ],
    )


def test_auto_configurator_installed_properly():
    # https://stackoverflow.com/a/65034142
    with patch.dict(sys.modules, {k: None for k in sys.modules if "pymdownx" in k}):
        md = Markdown(
            extensions=[
                "rst_in_md",
                "attr_list",
                "fenced_code",
            ],
        )
        assert "rst-in-md" in md.preprocessors
        assert "rst-in-md-auto-configurator" not in md.preprocessors

    md = Markdown(
        extensions=[
            "attr_list",
            "fenced_code",
            "pymdownx.superfences",
        ],
    )
    assert "rst-in-md" not in md.preprocessors
    assert "rst-in-md-auto-configurator" not in md.preprocessors


def test_load_extension_with_pymdownx(md):
    assert "rst-in-md" in md.preprocessors
    assert "rst-in-md-auto-configurator" in md.preprocessors
    assert md.preprocessors.get_index_for_name("rst-in-md-auto-configurator") == 0

    assert md.preprocessors["fenced_code_block"].config.get("custom_fences", []) == []

    md.convert(source="placeholder")

    with pytest.raises(KeyError):
        md.preprocessors["rst-in-md"]

    configs = md.preprocessors["fenced_code_block"].config.get("custom_fences", [])
    languages = ["rst", "rest", "restructuredtext"]
    assert len(configs) == len(languages)
    for config in configs:
        assert config["name"] in languages
        assert config["class"] == "rst-in-md"
        assert config["format"] == superfence_formatter
        assert config["validator"] == superfence_validator

    ext = None
    for _ext in md.registeredExtensions:
        if isinstance(_ext, SuperFencesCodeExtension):
            ext = _ext
            break

    assert ext is not None
    assert len(ext.superfences) == 4  # rst, rest, restructuredtext, built-in
    for fence in ext.superfences:
        if fence.get("name") == "superfences":
            continue
        assert fence.get("name") in languages


def test_load_extension_without_pymdownx(md):
    md = Markdown(
        extensions=[
            "rst_in_md",
            "attr_list",
            "fenced_code",
        ],
    )

    assert "rst-in-md" in md.preprocessors
    assert "rst-in-md-auto-configurator" in md.preprocessors
    assert md.preprocessors.get_index_for_name("rst-in-md-auto-configurator") == 0

    md.convert(source="placeholder")

    assert "rst-in-md" in md.preprocessors


def test_superfence_inside_admonition(md):
    plain = dedent("""
    ```rst
    +------------------------+------------+----------+----------+
    | Header row, column 1   | Header 2   | Header 3 | Header 4 |
    | (header rows optional) |            |          |          |
    +========================+============+==========+==========+
    | body row 1, column 1   | column 2   | column 3 | column 4 |
    +------------------------+------------+----------+----------+
    | body row 2             | Cells may span columns.          |
    +------------------------+------------+---------------------+
    | body row 3             | Cells may  | - Table cells       |
    +------------------------+ span rows. | - contain           |
    | body row 4             |            | - body elements.    |
    +------------------------+------------+---------------------+
    ```
    """).strip("\n")

    admonition = dedent("""
    !!! note
        ```rst
        +------------------------+------------+----------+----------+
        | Header row, column 1   | Header 2   | Header 3 | Header 4 |
        | (header rows optional) |            |          |          |
        +========================+============+==========+==========+
        | body row 1, column 1   | column 2   | column 3 | column 4 |
        +------------------------+------------+----------+----------+
        | body row 2             | Cells may span columns.          |
        +------------------------+------------+---------------------+
        | body row 3             | Cells may  | - Table cells       |
        +------------------------+ span rows. | - contain           |
        | body row 4             |            | - body elements.    |
        +------------------------+------------+---------------------+

        ```
    """).strip("\n")

    converted_plain = md.convert(source=plain)
    converted_admonition = md.convert(source=admonition)

    assert converted_plain in converted_admonition



"""Module for converting restructured text to html in a markdown compatible manner."""

import io
from contextlib import redirect_stderr

import docutils.core
from bs4 import BeautifulSoup, Tag, formatter

LANGUAGES = ["rst", "rest", "restructuredtext"]
ATTRIBUTES_TO_STRIP = ["class", "id", "name", "style", "border"]
BS4_FORMATTER = formatter.HTMLFormatter(indent=2)


def _rst_to_soup(rst: str) -> BeautifulSoup:
    """Convert reStructuredText to a BeautifulSoup object.

    This will convert the reStructuredText to HTML using docutils. The HTML is then
    converted to a BeautifulSoup object and returned.

    Errors and warnings are captured gracefully and raised at the end.

    This function is heavily inspired by this [rst2html](https://github.com/andrewpetrochenkov/rst2html.py/blob/b66942f16e93d7260748ecc90867c55a4bb3236d/rst2html/__init__.py)
    implementation.

    Args:
        rst (str): The reStructuredText to convert.

    Raises:
        ValueError: If there are any errors or warnings during the conversion.

    Returns:
        BeautifulSoup: The converted reStructuredText.
    """
    kwargs = {
        "writer_name": "html",
        "settings_overrides": {
            "_disable_config": True,
            "report_level": 2,
        },
    }

    with io.StringIO() as target, redirect_stderr(target):
        parts = docutils.core.publish_parts(rst, **kwargs)
        warning = target.getvalue().strip()

    if warning:
        msg = f"Failed to convert restructured text:\n\n{warning}"
        raise ValueError(msg)

    return BeautifulSoup(parts.get("body"), features="html.parser")


def _strip_attributes(soup: BeautifulSoup) -> BeautifulSoup:
    """Remove specific attributes from the soup.

    This will remove all attributes from the top level tags, and will also remove
    some attributes from the descendants. This took heavy inspiration from this
    [StackOverflow answer](https://stackoverflow.com/a/9045719).

    Args:
        soup (BeautifulSoup): Input soup to remove attributes from.

    Returns:
        BeautifulSoup: Same soup with attributes removed.
    """
    # Remove attributes from the top level tags
    for tag in soup.contents:
        if isinstance(tag, Tag):
            tag.attrs = {}

    # Remove specific attributes from the descendants
    for tag in soup.descendants:
        if isinstance(tag, Tag):
            tag.attrs = {
                key: value
                for key, value in tag.attrs.items()
                if key not in ATTRIBUTES_TO_STRIP
            }
    return soup


def rst_to_soup(rst: str) -> BeautifulSoup:
    """Convert restructured text to html in a manner that is compatible with markdown.

    Args:
        rst (str): Raw restructured text to convert to html.

    Returns:
        str: Html converted from restructured text.
    """
    soup = _rst_to_soup(rst)
    return _strip_attributes(soup)



from textwrap import dedent

import pytest

from rst_in_md import RestructuredTextInMarkdownPreProcessor


@pytest.fixture()
def preprocessor():
    return RestructuredTextInMarkdownPreProcessor()


def test_run(preprocessor):
    unprocessed = dedent("""
    ```rst
    Section 1
    ```

    Not rst

    ```restructuredtext
    Section 2
    ```

    Not restructuredtext

    ```rest
    Section 3
    ```

    Not rest
    """).strip("\n")

    processed = "\n".join(preprocessor.run(lines=unprocessed.split("\n")))

    for i in range(1, 4):
        assert f"<p>\n  Section {i}\n</p>" in processed



"""The RestructuredTextInMarkdown extension."""

from markdown import Extension, Markdown

from rst_in_md.processor import RestructuredTextInMarkdownPreProcessor
from rst_in_md.superfence import RestructuredTextInMarkdownAutoConfigurator


class RestructuredTextInMarkdown(Extension):
    """Extension to convert restructured text to html in markdown."""

    @staticmethod
    def _pymdownx_installed() -> bool:
        try:
            import pymdownx.superfences  # noqa: F401
        except ImportError:
            return False
        return True

    def extendMarkdown(self, md: Markdown) -> None:  # noqa: N802
        """Register the RestructuredTextInMarkdownPreProcessor.

        Register the
        [RestructuredTextInMarkdownPreProcessor](extension.md#rst_in_md.RestructuredTextInMarkdownPreProcessor)
        with the markdown instance. This allows the preprocessor to be used when
        markdown is rendered to html.

        The priority of the preprocessor is set to `27`. This is higher than
        the [fenced_code_block](https://github.com/Python-Markdown/markdown/blob/33359faa385f59b84cd87df5f4b0996055a482e2/markdown/extensions/fenced_code.py#L50)
        preprocessor, so that `rst` blocks are processed beforehand. But it is
        lower than [normalize_whitespace](https://github.com/Python-Markdown/markdown/blob/33359faa385f59b84cd87df5f4b0996055a482e2/markdown/preprocessors.py#L40)
        so that the rst blocks can be processed in a similar manner to code blocks.

        Also register the
        [RestructuredTextInMarkdownAutoConfigurator](superfence.md#rst_in_md.RestructuredTextInMarkdownAutoConfigurator)
        if `pymdownx.superfences` is installed.

        Args:
            md (Markdown): The Markdown instance.
        """
        md.preprocessors.register(
            RestructuredTextInMarkdownPreProcessor(md),
            "rst-in-md",
            27,
        )

        if self._pymdownx_installed():
            md.preprocessors.register(
                RestructuredTextInMarkdownAutoConfigurator(md),
                "rst-in-md-auto-configurator",
                300,
            )



import pytest
from bs4 import Tag

from rst_in_md import rst_to_soup


def test_fail():
    text = ".. table:: \n this is going to fail"
    with pytest.raises(ValueError, match="Failed to convert restructured text"):
        rst_to_soup(text)


def test_convert_multiple():
    text = """
.. table:: simple table
   :widths: auto

   =====  =====
     A    not A
   =====  =====
   False  True
   True   False
   =====  =====

.. list-table:: list table
   :widths: 15 10 30
   :header-rows: 1

   * - Treat
     - Quantity
     - Description
   * - Albatross
     - 2.99
     - On a stick!
   * - Crunchy Frog
     - 1.49
     - If we took the bones out, it wouldn't be
       crunchy, now would it?
   * - Gannet Ripple
     - 1.99
     - On a stick!
"""

    soup = rst_to_soup(text)

    tags = [t for t in soup.contents if isinstance(t, Tag)]
    assert len(tags) == 2

    table, list_table = tags

    assert table.name == "table"
    assert len(table.find_all("tr")) == 3
    assert len(table.find_all("caption")) == 1
    assert table.find_all("caption")[0].text == "simple table"

    assert list_table.name == "table"
    assert len(list_table.find_all("tr")) == 4
    assert len(list_table.find_all("caption")) == 1
    assert list_table.find_all("caption")[0].text == "list table"



"""PyMdown Extensions Custom Superfence for `rst_in_md`."""

import logging
from functools import partial

from markdown import Markdown
from markdown.preprocessors import Preprocessor

from rst_in_md.conversion import BS4_FORMATTER, LANGUAGES, rst_to_soup

logger = logging.getLogger(__name__)
logging.basicConfig(level=logging.INFO)


def superfence_formatter(  # noqa: D417
    source: str,
    language: str,  # noqa: ARG001
    css_class: str,  # noqa: ARG001
    options: dict,  # noqa: ARG001
    md: Markdown,  # noqa: ARG001
    **kwargs: dict,  # noqa: ARG001
) -> str:
    """Convert superfenced reStructuredText to html.

    This function will convert the reStructuredText to html using the same method as
    the standard python markdown extension.

    !!! note "Unused Arguments"
        This function is passed a few arguments that are not used. It must adhere to
        [the required signature](https://facelessuser.github.io/pymdown-extensions/extensions/superfences/#formatters)
        set by `pymdownx.superfences`.

    Args:
        source (str): Language of the superfence.
        language (str): Language of the superfence _(required, but not used)_.
        css_class (str): CSS class of the superfence _(required, but not used)_.
        options (dict): Options of the superfence _(required, but not used)_.
        md (Markdown): The markdown instance _(required, but not used)_.

    Returns:
        str: The converted html.
    """
    return rst_to_soup(source).prettify(formatter=BS4_FORMATTER)


def superfence_validator(
    language: str,
    inputs: dict,
    options: dict,
    attrs: dict,
    md: Markdown,  # noqa: ARG001
) -> bool:
    """Validate that the superfence should be processed.

    This function will validate that the superfence should be processed by `rst-in-md`.
    This includes:

    * Checking if the language is supported.
    * Checking if the `rst-in-md` attribute is set to `false` or not.
    * Checking if any options or attributes are passed.

    !!! note "Unused Arguments"
        `md` is passed to this function but is not used. It must adhere to
        [the required signature](https://facelessuser.github.io/pymdown-extensions/extensions/superfences/#validators)
        set by `pymdownx.superfences`.

    Args:
        language (str): Language of the superfence.
        inputs (dict): All the parsed options/attributes of the superfence.
        options (dict): A dictionary to which all valid options should be assigned to.
        attrs (dict): A dictionary to which all valid attributes should be assigned to.
        md (Markdown): the markdown instance _(required, but not used)_.

    Returns:
        bool: If the superfence should be processed or not.
    """
    if language not in LANGUAGES:
        msg = f"language '{language}' is not supported."
        logging.error(msg)
        return False

    allowed = {"rst-in-md"}
    if (keys := set(inputs.keys())) > allowed:
        msg = f"keys '{keys - allowed}' are not supported."
        logging.error(msg)
        return False

    if inputs.get("rst-in-md") == "false":
        logging.info("rst-in-md is set to false.")
        return False

    if len(options) > 0:
        logging.error("options are not supported.")
        return False

    if len(attrs) > 0:
        logging.error("attrs are not supported.")
        return False

    return True


class RestructuredTextInMarkdownAutoConfigurator(Preprocessor):
    """Preprocessor to adapt `rst-in-md` to work with `pymdownx.superfences`."""

    initialized = False

    def superfences_installed(self) -> bool:
        """Check if the `pymdownx.superfences` extension is installed.

        Returns:
            bool: If the extension is installed or not.
        """
        try:
            from pymdownx.superfences import SuperFencesBlockPreprocessor
        except ImportError:
            return False

        return isinstance(
            self.md.preprocessors["fenced_code_block"],
            SuperFencesBlockPreprocessor,
        )

    @staticmethod
    def construct_fence_config(language: str) -> dict:
        """Create a fence configuration dictionary for `pymdownx.superfences`.

        Create a fence configuration for `pymdownx.superfences` with the given language,
        in the given [structure](https://github.com/facelessuser/pymdown-extensions/blob/cd7c704487a3a79b6619bfcd0c6af83104d630a8/pymdownx/superfences.py#L273-L276).
        This configuration will be read by `pymdownx.superfences` to determine how to
        process the superfence.

        Args:
            language (str): Language of the superfence.

        Returns:
            dict: Dictionary of the fence configuration.
        """
        return {
            "name": language,
            "class": "rst-in-md",
            "format": superfence_formatter,
            "validator": superfence_validator,
        }

    @staticmethod
    def construct_superfence(language: str) -> dict:
        """Create a superfence dictionary for `pymdownx.superfences`.

        Create a superfence dict for `pymdownx.superfences` with the given language,
        in the given [structure](https://github.com/facelessuser/pymdown-extensions/blob/cd7c704487a3a79b6619bfcd0c6af83104d630a8/pymdownx/superfences.py#L240-L245).
        This dictionary will be appended to `SuperFencesCodeExtension().superfences`.

        Args:
            language (str): Language of the superfence.

        Returns:
            dict: Dictionary of the superfence for `pymdownx.superfences`.

        Raises:
            ImportError: pymdown-extensions is not properly installed.
        """
        try:
            from pymdownx.superfences import _formatter, _test, _validator
        except ImportError as e:
            msg = "pymdown-extensions is not properly installed."
            raise ImportError(msg) from e

        return {
            "name": language,
            "test": partial(_test, test_language=language),
            "formatter": partial(
                _formatter,
                class_name="rst-in-md",
                _fmt=superfence_formatter,
            ),
            "validator": partial(
                _validator,
                validator=superfence_validator,
            ),
        }

    def inject_custom_configs(self) -> None:
        """Add custom fence configs to `pymdownx.superfences`, if not already present.

        Raises:
            ImportError: pymdown-extensions is not properly installed.
            ValueError: SuperFencesCodeExtension not found.
        """
        try:
            from pymdownx.superfences import SuperFencesCodeExtension
        except ImportError as e:
            msg = "pymdown-extensions is not properly installed."
            raise ImportError(msg) from e

        registered = self.md.registeredExtensions
        extensions = [e for e in registered if isinstance(e, SuperFencesCodeExtension)]
        if len(extensions) != 1:
            msg = "Unable to find SuperFencesCodeExtension."
            raise ValueError(msg)
        ext = extensions[0]

        config = self.md.preprocessors["fenced_code_block"].config  # pyright: ignore[reportAttributeAccessIssue]
        custom_fences = config.get("custom_fences", [])
        for language in LANGUAGES:
            if (fence := self.construct_fence_config(language)) not in custom_fences:
                custom_fences.append(fence)
                ext.superfences.append(self.construct_superfence(language))

        config["custom_fences"] = custom_fences

    def run(self, lines: list[str]) -> list[str]:
        """Auto-configure `pymdownx.superfences` if installed.

        This method will check if `pymdownx.superfences` is installed. If it is, it will
        deregister `rst-in-md`, since `pymdownx.superfences` will handle the fenced code
        blocks. It will also provide the custom fence configurations needed for
        `pymdownx.superfences` to properly process the fenced code.

        !!! question "Why is this a preprocessor?"

            This preprocessor will not actually process any markdown, even if it is
            called for each markdown file. It much be run after all extensions have been
            initialized.

        Args:
            lines (list[str]): Input lines _(required, but not used)_.

        Returns:
            list[str]: Identical as the input lines.
        """
        if not self.initialized and self.superfences_installed():
            self.md.preprocessors.deregister("rst-in-md")
            self.inject_custom_configs()
            self.initialized = True

        return lines



# ruff: noqa: E501
from textwrap import dedent

from bs4 import BeautifulSoup

from rst_in_md import rst_to_soup


def test_simple_table():
    text = """
.. table::
   :widths: auto

   =====  =====  ======
      Inputs     Output
   ------------  ------
     A      B    A or B
   =====  =====  ======
   False  False  False
   True   False  True
   False  True   True
   True   True   True
   =====  =====  ======
    """
    rst_soup = rst_to_soup(dedent(text).strip("\n"))

    soup = BeautifulSoup(
        """
        <table>
        <thead valign="bottom">
            <tr><th colspan="2">Inputs</th><th>Output</th></tr>
            <tr><th>A</th><th>B</th><th>A or B</th></tr>
        </thead>
        <tbody valign="top">
            <tr><td>False</td><td>False</td><td>False</td></tr>
            <tr><td>True</td><td>False</td><td>True</td></tr>
            <tr><td>False</td><td>True</td><td>True</td></tr>
            <tr><td>True</td><td>True</td><td>True</td></tr>
        </tbody>
        </table>
        """,
        features="html.parser",
    )

    assert rst_soup.prettify() == soup.prettify()


def test_grid_table():
    text = """
+------------------------+------------+----------+----------+
| Header row, column 1   | Header 2   | Header 3 | Header 4 |
| (header rows optional) |            |          |          |
+========================+============+==========+==========+
| body row 1, column 1   | column 2   | column 3 | column 4 |
+------------------------+------------+----------+----------+
| body row 2             | Cells may span columns.          |
+------------------------+------------+---------------------+
| body row 3             | Cells may  | - Table cells       |
+------------------------+ span rows. | - contain           |
| body row 4             |            | - body elements.    |
+------------------------+------------+---------------------+
    """
    rst_soup = rst_to_soup(dedent(text).strip("\n"))
    soup = BeautifulSoup(
        """
        <table>
            <colgroup><col width="43%"/><col width="21%"/><col width="18%"/><col width="18%"/></colgroup>
            <thead valign="bottom">
                <tr><th>Header row, column 1\n(header rows optional)</th><th>Header 2</th><th>Header 3</th><th>Header 4</th></tr>
            </thead>
            <tbody valign="top">
                <tr><td>body row 1, column 1</td><td>column 2</td><td>column 3</td><td>column 4</td></tr>
                <tr><td>body row 2</td><td colspan="3">Cells may span columns.</td></tr>
                <tr><td>body row 3</td><td rowspan="2">Cells may\nspan rows.</td><td colspan="2" rowspan="2"><ul><li>Table cells</li><li>contain</li><li>body elements.</li></ul></td></tr>
                <tr><td>body row 4</td></tr>
            </tbody>
        </table>
        """,
        features="html.parser",
    )

    assert rst_soup.prettify() == soup.prettify()


def test_auto_widths_table():
    text = """
.. list-table::
  :widths: auto
  :header-rows: 1

  * - Treat
    - Quantity
    - Description
  * - Albatross
    - 2.99
    - On a stick!
  * - Crunchy Frog
    - 1.49
    - If we took the bones out, it wouldn't be crunchy, now would it?
  * - Gannet Ripple
    - 1.99
    - On a stick!
    """

    rst_soup = rst_to_soup(text)

    soup = BeautifulSoup(
        """
        <table>
        <thead valign="bottom">
            <tr><th>Treat</th><th>Quantity</th><th>Description</th></tr>
        </thead>
        <tbody valign="top">
            <tr><td>Albatross</td><td>2.99</td><td>On a stick!</td></tr>
            <tr><td>Crunchy Frog</td><td>1.49</td><td>If we took the bones out, it wouldn't be crunchy, now would it?</td></tr>
            <tr><td>Gannet Ripple</td><td>1.99</td><td>On a stick!</td></tr>
        </tbody>
        </table>
        """,
        features="html.parser",
    )

    assert rst_soup.prettify() == soup.prettify()


def test_fixed_widths_table():
    text = """
.. list-table::
  :widths: 15 10 30
  :header-rows: 1

  * - Treat
    - Quantity
    - Description
  * - Albatross
    - 2.99
    - On a stick!
  * - Crunchy Frog
    - 1.49
    - If we took the bones out, it wouldn't be crunchy, now would it?
  * - Gannet Ripple
    - 1.99
    - On a stick!
    """

    rst_soup = rst_to_soup(text)

    soup = BeautifulSoup(
        """
        <table>
        <colgroup><col width="27%"/><col width="18%"/><col width="55%"/></colgroup>
        <thead valign="bottom">
            <tr><th>Treat</th><th>Quantity</th><th>Description</th></tr>
        </thead>
        <tbody valign="top">
            <tr><td>Albatross</td><td>2.99</td><td>On a stick!</td></tr>
            <tr><td>Crunchy Frog</td><td>1.49</td><td>If we took the bones out, it wouldn't be crunchy, now would it?</td></tr>
            <tr><td>Gannet Ripple</td><td>1.99</td><td>On a stick!</td></tr>
        </tbody>
        </table>
        """,
        features="html.parser",
    )

    assert rst_soup.prettify() == soup.prettify()



"""This module is the entry point for the RestructuredTextInMarkdown extension."""

from markdown.extensions import Extension

from rst_in_md.conversion import rst_to_soup
from rst_in_md.extension import RestructuredTextInMarkdown
from rst_in_md.processor import RestructuredTextInMarkdownPreProcessor
from rst_in_md.superfence import (
    RestructuredTextInMarkdownAutoConfigurator,
    superfence_formatter,
    superfence_validator,
)

__all__ = [
    "rst_to_soup",
    "RestructuredTextInMarkdown",
    "RestructuredTextInMarkdownPreProcessor",
    "RestructuredTextInMarkdownAutoConfigurator",
    "superfence_formatter",
    "superfence_validator",
]

__version__ = "0.2.1"


def makeExtension(**kwargs: dict) -> Extension:  # noqa: N802
    """Return an instance of the RestructuredTextInMarkdown extension.

    This function is specified by the markdown package, so that it can properly load the
    extension. You can find more information about this function in the [markdown documentation](https://python-markdown.github.io/extensions/api/#dot_notation).

    Returns:
        Extension: An instance of the RestructuredTextInMarkdown extension.
    """
    return RestructuredTextInMarkdown(**kwargs)
